@@ -1,7 +1,5 @@
-# Include source function library.
-. /usr/share/scap-security-guide/remediation_functions
 
-# platform = Red Hat Enterprise Linux 7,Red Hat Enterprise Linux 8,multi_platform_rhv
+# platform = multi_platform_all
 if [ "$(getconf LONG_BIT)" = "32" ] ; then
   #
   # Set runtime for kernel.exec-shield
@@ -12,7 +10,7 @@ if [ "$(getconf LONG_BIT)" = "32" ] ; then
   # If kernel.exec-shield present in /etc/sysctl.conf, change value to "1"
   #	else, add "kernel.exec-shield = 1" to /etc/sysctl.conf
   #
-  replace_or_append '/etc/sysctl.conf' '^kernel.exec-shield' '1' '@CCENUM@'
+  {{{ bash_replace_or_append('/etc/sysctl.conf', '^kernel.exec-shield', '1', '@CCENUM@') }}}
 fi
 
 if [ "$(getconf LONG_BIT)" = "64" ] ; then
